FUNCTION LASCO_READFITS, filename0, header, FITS_HDR=FITS_HDR, NO_IMG=NO_IMG, REFCOORD=REFCOORD, $
	SILENT=silent, CACHE=cache
;+
; Project     : SOHO - LASCO
;                   
; Name        : LASCO_READFITS
;               
; Purpose     : Read a LASCO FITS file to obtain an image and header array.
;               
; Explanation : This routine calls the IDL Astronomy Library routine READFITS
;		to read the LASCO FITS file.  It then fills in a LASCO header
;		structure with the header information.
;               
; Use         : IDL> image = LASCO_READFITS('filename' [,header])
;    
; Inputs      : filename	Name of the LASCO FITS file.
;               
; Outputs     : image		The image array.
;               
; Opt. Outputs: header		The image header can be returned either as a
;				LASCO header structure or a STRARR (FITS header).
;               
; Keywords    : FITS_HDR	The header is normally returned as a LASCO header
;				structure.  Set this keyword to return the header
;				as a STRARR (FITS header).
;
;               NO_IMG  	Set this keyword if you only want the header returned.
;
;               REFCOORD  	Set this keyword to define CRPIX if needed.
;               SILENT  	Set this keyword to turn off printing of remarks
;		CACHE		Set equal to list of fits files; if set, then will copy 
;				at most 25 files to /tmp and uses these copies 
;
; Calls       : READFITS, HEADFITS
;
; Category    : Data_Handling, I_O
;               
; Prev. Hist. : None.
;
; Written     : Scott Paswaters, NRL, Feb. 1996.
;               
; Modified    : 
;
; Version     : 
;		1996/09/27  SEP 	Added REFCOORD
;		RAH, 5/23/97, added error handling for call to READFITS
;		RAH, 8/28/98, added handling of MLO/MK3 images
;		NBR, 12/1999, retry READFITS if it fails
;		NBR, 01/2000, Repeat READFITS max 5 times or until it succeeds
;		NBR, 02/2000, Add 2nd catch function and GOTO function for null image; 
;				IF incr GT 5, return -1
;		NBR, 05/2000, Also repeat for null hdr
;		RAH, 7/28/00, added handling of MLO/MK4 images
;		NBR, 09/06/00, Check for .gz FITS if unable to find file
;	NBR, 12/04/00, Fix error condition
;		DW,  12/04/00, Add SILENT Keyword
;		NBR, 01/17/01, Add help statements
;	NBR, 02/12/01, Convert all headers to structure, regardless if LASCO or MK 
;	NBR, 11/30/01 - Replace TRIM with STRTRIM
;	NBR, 01/03/02 - Add CACHE keyword
;	NBR, 01/07/02 - Edit SILENT usage
;
;
;	01/10/02, @(#)lasco_readfits.pro	1.13 LASCO IDL LIBRARY
;-            

;______________________________________________________________________________
;

;
;  The catch statement and then the following IF statement MUST be
;  together.  CATCH returns control to the following statement when
;  it detects an error.
;
incr=0
t1 = systime(1)
CATCH,bad

IF keyword_set(CACHE) THEN BEGIN
	break_file,filename0,bflog,dir,fname,ext
	filename='/tmp/'+fname+ext
	IF file_stat(filename,/size) LT 131000 THEN BEGIN
; If filename is nonexistent or smaller than equivalent of 256x256 image....
		cmd = '/bin/rm -fr /tmp/*.fts'
		print,cmd
		spawn, cmd,/SH
		n=n_elements(cache)
		i0=where(cache EQ filename0,nii)
		i0=i0[0]
		in = (i0+24) < (n-1)
		FOR i=i0,in DO BEGIN
			cmd='cp '+cache[i]+' /tmp'
			print,cmd
			spawn,cmd,/SH
		ENDFOR
	ENDIF
ENDIF ELSE filename=filename0


IF (bad NE 0)  THEN BEGIN
   PRINT,'%%LASCO_READFITS:  Error in HEADFITS/READFITS'
   header = -1
   img = -1
   HELP,bad
   print,!err_string,!error
   RETURN,img
ENDIF
IF KEYWORD_SET(NO_IMG) THEN BEGIN
      hdr = HEADFITS(filename)	;, SILENT=silent)
         img = -1
ENDIF ELSE BEGIN
      img = READFITS(filename, hdr, SILENT=silent)
ENDELSE
CATCH,/cancel

CATCH,bad2

IF (bad2 NE 0)  THEN BEGIN
   PRINT,'%%LASCO_READFITS:  Error in HEADFITS/READFITS'
   header = -1
   img = -1
   help,bad2
   print,!err_string,!error
   RETURN,img
ENDIF

IF datatype(hdr) EQ 'UND' THEN hdr=-1
IF img(0) EQ -1 and hdr(0) EQ '-1' THEN BEGIN
	print,'Trying '+filename+'.gz....'
	img = READFITS(filename+'.gz',hdr, SILENT=silent)
ENDIF

IF img(0) EQ -1 and hdr(0) EQ '-1' THEN BEGIN
   REPEAT BEGIN
	IF n_elements(hdr) GT 1 THEN IF fxpar(hdr,'NAXIS1') LE 0 THEN GOTO, going_on
	print,'Waiting before trying again....'
	wait,5
	img = READFITS(filename,hdr,SILENT=silent) 
       	incr = incr+1
   ENDREP UNTIL img(0) GE 0 OR incr GT 5
   IF incr GT 5 THEN BEGIN
   	header = -1
   	img = -1
   	RETURN,img
   ENDIF
ENDIF		; NBR, 01/2000
CATCH,/cancel

going_on:

   IF KEYWORD_SET(REFCOORD) THEN BEGIN
      tmp = FXPAR(hdr, 'CTYPE1')
      IF (DATATYPE(tmp) NE 'STR') THEN REDUCE_REFCOORD, hdr, '0.5'
      ind = WHERE(STRTRIM(hdr,2) NE '')
      hdr = hdr(ind)
   ENDIF

   IF KEYWORD_SET(FITS_HDR) THEN BEGIN
      header = hdr
   ENDIF ELSE BEGIN
      ;** convert the fits header strarr to a lasco header structure
      val = FXPAR(hdr, 'INSTRUME')
      IF ( (STRTRIM(val,2) EQ 'LASCO') OR (STRTRIM(val,2) EQ 'EIT') ) THEN BEGIN
         header = LASCO_FITSHDR2STRUCT(hdr)
      ENDIF ELSE BEGIN
         val = STRUPCASE(STRTRIM(FXPAR(hdr,'TELESCOP'),2))
         IF (val EQ 'MK3')  OR (val EQ 'MK4')  THEN BEGIN
            header = MLO_FITSHDR2STRUCT(hdr)
            header.filename = filename
         ENDIF ELSE BEGIN
         PRINT, '%%LASCO_READFITS:  File: ', filename, " is not a LASCO fits file.  Header structure may be inaccurate."
         header = LASCO_FITSHDR2STRUCT(hdr)
         ENDELSE
      ENDELSE
   ENDELSE

IF NOT KEYWORD_SET (SILENT) THEN $
PRINT,'%%LASCO_READFITS took ',STRTRIM(STRING(systime(1)-t1)),' seconds.'  ; **NBR, 1/10/00
   RETURN, img

BAD:
   RETURN,-1

END
