;======================== header ===============================================
;+
; NAME:
;   pics2zip
; PURPOSE:
;   make zip files for Solar Weather Browser for MLSO PICS (H-alpha) images.
; CATEGORY:
;   SWB version 1.00
; CALLING SEQUENCE:
;   (meant to be used in batchmode)
; KEYWORD PARAMETERS:
;   /logfile: to print diagnostics to a logfile (default=stdout=screen)
;   date: yyyymmdd
; MODIFICATION HISTORY:
; Created by Andrew L. Stanger, ROB, 04 Jun 2007: Adapted from "mk42zip.pro".
;-
;======================== end header ===========================================

PRO pics2zip, logfile=logfile, date=date

; ------------------------------------------------------------------------------
; Procedure name.
; ------------------------------------------------------------------------------

myname = 'pics2zip'

; ------------------------------------------------------------------------------
; Input data directory.
; ------------------------------------------------------------------------------

set_swbs
datadir = !swbs.data_dir + 'MLSO/pics/' + date + '/'
print, 'datadir: ', datadir

; ------------------------------------------------------------------------------
; Open Log file.
; ------------------------------------------------------------------------------

logfilename = !swbs.log_dir + myname + '/' + myname + '.log'

IF (keyword_set (logfile)) THEN $
   openw, log, logfilename, /get_lun, /append, bufsize=0 $
ELSE log = -1

; ------------------------------------------------------------------------------
; Say hello.
; ------------------------------------------------------------------------------

printf, log, ' >>>> ' + myname + '.pro started at ' + systime ()

; ------------------------------------------------------------------------------
; Reference information.
; ------------------------------------------------------------------------------

outdim        = !swbs.outdim			; 512
ref_solradius = !swbs.ref_solradius		; 184
ref_xcenter   = !swbs.ref_xcenter		; 256
ref_ycenter   = !swbs.ref_ycenter		; 256

; ------------------------------------------------------------------------------
; Get list of good images
; ------------------------------------------------------------------------------

sdir = datadir + '*dpm.asc.fts*'
print, 'sdir: ', sdir

;list = file_search (datadir + '*pics.asc.fts*')

spawn, 'ls ' + sdir, list

; ------------------------------------------------------------------------------
; Continue only if list is not empty.
; ------------------------------------------------------------------------------

IF (list [0] EQ ' ') THEN printf, log, '  No images available for this period' $
ELSE BEGIN ;{ ------- list not empty -------------

   FOR i= 0, n_elements (list) - 1 DO	$
   BEGIN ;{

   fitsfile = list [i]
   print, 'fitsfile: ', fitsfile

; ------------------------------------------------------------------------------
; Report starting processing image
; ------------------------------------------------------------------------------

      printf, log, ''
      printf, log, 'Processing image ', list[i]
      printf, log, ''

; ------------------------------------------------------------------------------
; Read image and header.
; ------------------------------------------------------------------------------

      image = mlso_readfits (list [i], index, /silent)

      IF (size (image, /n_dim) EQ 0) THEN	$
      BEGIN
         printf, log, "pics2zip: Skipping file " + readdir + list [i]
         continue
      ENDIF

; ------------------------------------------------------------------------------
; Date and Time.
; ------------------------------------------------------------------------------

      pics_date = index.date_obs
      pics_time = index.time_obs
      date     = index.date_obs + "T" + index.time_obs

      year     = strmid (pics_date,  0, 4)
      month    = strmid (pics_date,  5, 2)
      day      = strmid (pics_date,  8, 2)

      hours    = strmid (pics_time,  0, 2)
      mins     = strmid (pics_time,  3, 2)
      sec      = strmid (pics_time,  6, 2)

; ------------------------------------------------------------------------------
; Solar radius in arcsec.
; ------------------------------------------------------------------------------

      sol_rad = index.rsun

; ------------------------------------------------------------------------------
; Image brightness min/max values.
; ------------------------------------------------------------------------------

      dispmin = index.dispmin		; display minimum.
      dispmax = index.dispmax		; display maximum.

; ------------------------------------------------------------------------------
; Re-size image.
; ------------------------------------------------------------------------------
      image   = image > 0
      image   = congrid (image, outdim, outdim)		; Re-size image.

; ------------------------------------------------------------------------------
; Adjust pixel brightness, using dispmin & dispmax values.
; ------------------------------------------------------------------------------

      image   = bytscl (image, dispmin, dispmax)	; Re-scale brightness.
      mask_im = where (image LE min (image))		; Clip dim values.
      image [mask_im] = 0  

; ------------------------------------------------------------------------------
; Create outdir, outfn, color_table_code.
; ------------------------------------------------------------------------------

      outfn = 'mlso_pics_' + year + month + day + '_' + hours + mins + sec
      outdir = !swbs.out_dir + 'MLSO/' + year + '/' + month + '/' + day + '/'
      data_type = 'C'
      data_name = 'MLSO H-alpha'
      color_table_code = '001' 

; ------------------------------------------------------------------------------
; SWB's standard Field Of View (FOV).
; ------------------------------------------------------------------------------

      swb_fov = sol_rad * outdim / ref_solradius  

; ------------------------------------------------------------------------------
; Calculate the ratio of COR's FOV to the standard SWB's FOV.
; ------------------------------------------------------------------------------

      pics_fov = index.naxis1 * index.cdelt1 
      fov_ratio = pics_fov / swb_fov

; ------------------------------------------------------------------------------
; Create zip file.
; ------------------------------------------------------------------------------

      write2zip, outfn, outdir, image, index, data_type, data_name, $
               color_table_code, date, fov_ratio=fov_ratio, /logfile
      ;cd, !swbs.source_dir + 'bin/' + myname
      printf, log, '  processed pics image ', list [i]

; ------------------------------------------------------------------------------

   ENDFOR ;} i
ENDELSE	;} ------- list not empty -------------

; ------------------------------------------------------------------------------
; Say bye-bye.
; ------------------------------------------------------------------------------

spawn, 'date', fulldate   
printf, log, ' <<<< ' + myname + ' finished at ' + fulldate 

; ------------------------------------------------------------------------------
; Close log file, if needed.
; ------------------------------------------------------------------------------

IF (keyword_set (logfile)) THEN free_lun, log   
END
