;======================== header ===============================================
;+
; NAME:
;   picsf2zip
; PURPOSE:
;   Make zip files for Solar Weather Browser:
;   MLSO PICS (H-alpha Disc Flat) images.
; CATEGORY:
;   SWB version 1.00
; CALLING SEQUENCE:
;   (meant to be used in batchmode)
; KEYWORD PARAMETERS:
;   /logfile: to print diagnostics to a logfile (default=stdout=screen)
;   date: yyyymmdd
; MODIFICATION HISTORY:
; Created by Andrew L. Stanger, ROB, 04 Jun 2007: Adapted from "mk42zip.pro".
; 07 Jun 2007 [ALS]; Add rotation to correct for solar P-angle.
; 21 Jun 2007 [ALS]: Set in/out directories for HAO/MLSO archive.
; 29 Jun 2007 [ALS]: Set pixels in outer FOV to zero.
;-
;======================== end header ===========================================

PRO picsf2zip, logfile=logfile, date=date

; ------------------------------------------------------------------------------
; Procedure name.
; ------------------------------------------------------------------------------

myname = 'picsf2zip'

; ------------------------------------------------------------------------------
; Get year, month & day from date parameter.
; ------------------------------------------------------------------------------

year  = strmid (date, 0, 4)
month = strmid (date, 4, 2)
day   = strmid (date, 6, 2)

; ------------------------------------------------------------------------------
; Input data directory.
; ------------------------------------------------------------------------------

set_swbs
;datadir = !swbs.data_dir + 'MLSO/pics/' + date + '/'

datadir = '/archive/acos/' + year + '/' + month + '/' + day + '/'
print, 'datadir: ', datadir

; ------------------------------------------------------------------------------
; Open Log file.
; ------------------------------------------------------------------------------

logfilename = !swbs.log_dir + myname + '/' + myname + '.log'

IF (keyword_set (logfile)) THEN $
   openw, log, logfilename, /get_lun, /append, bufsize=0 $
ELSE log = -1

; ------------------------------------------------------------------------------
; Say hello.
; ------------------------------------------------------------------------------

printf, log, ' >>>> ' + myname + '.pro started at ' + systime ()

; ------------------------------------------------------------------------------
; Reference information.
; ------------------------------------------------------------------------------

outdim        = !swbs.outdim			; 512
ref_solradius = !swbs.ref_solradius		; 184
ref_xcenter   = !swbs.ref_xcenter		; 256
ref_ycenter   = !swbs.ref_ycenter		; 256

window, 2, xs=outdim, ys=outdim, retain=2

; ------------------------------------------------------------------------------
; Get list of good images
; ------------------------------------------------------------------------------

sdir = datadir + '*dpm.asl.fts*'
print, 'sdir: ', sdir

;list = file_search (datadir + '*pics.asl.fts*')

spawn, 'ls ' + sdir, list

; ------------------------------------------------------------------------------
; Continue only if list is not empty.
; ------------------------------------------------------------------------------

IF (list [0] EQ ' ') THEN printf, log, '  No images available for this period' $
ELSE BEGIN ;{ ------- list not empty -------------

    FOR i= 0, n_elements (list) - 1 DO	$
    BEGIN ;{

    fitsfile = list [i]
    print, 'fitsfile: ', fitsfile

    ; --------------------------------------------------------------------------
    ; Report starting processing image
    ; --------------------------------------------------------------------------

    printf, log, ''
    printf, log, 'Processing image ', list[i]
    printf, log, ''

    ; --------------------------------------------------------------------------
    ; Read image and header.
    ; --------------------------------------------------------------------------

    img = mlso_readfits (list [i], index, /silent)

    IF (size (img, /n_dim) EQ 0) THEN	$
    BEGIN
        printf, log, "picsf2zip: Skipping file " + readdir + list [i]
        continue
    ENDIF

    ; --------------------------------------------------------------------------
    ; Date and Time.
    ; --------------------------------------------------------------------------

    pics_date = index.date_obs
    pics_time = index.time_obs
    date     = index.date_obs + "T" + index.time_obs

    year     = strmid (pics_date,  0, 4)
    month    = strmid (pics_date,  5, 2)
    day      = strmid (pics_date,  8, 2)

    hours    = strmid (pics_time,  0, 2)
    mins     = strmid (pics_time,  3, 2)
    sec      = strmid (pics_time,  6, 2)

    ; --------------------------------------------------------------------------
    ; Rotation angle & sun center.
    ; --------------------------------------------------------------------------

    rot_ang = -index.crota1		; rotation angle.
    xcen    = index.crpix1		; x-axis center.
    ycen    = index.crpix2		; y-axis center.

    print, 'rot_ang    : ', rot_ang
    print, 'xcen, ycen : ', xcen, ycen

    ; --------------------------------------------------------------------------
    ; Rotate image to correct for the P-angle.
    ; --------------------------------------------------------------------------

    img = rot (img, rot_ang, 1.0, xcen, ycen,  missing=0, /interp)

    ; --------------------------------------------------------------------------
    ; Re-size image.
    ; --------------------------------------------------------------------------

    icolor =   0 

    mag    = float (outdim) / float (index.naxis1)	; Magnification factor.
    oxcen  = index.crpix1 * mag				; x-axis sun center.
    oycen  = index.crpix2 * mag				; y-axis sun center.

    xdim   = index.naxis1
    ydim   = index.naxis2
    pixrs  = index.crradius		; pixels/Rsun
    opixrs = index.crradius * mag			; pixels/Rsun

    x1 = (xdim - 1024) / 2
    x2 = x1 + 1023
    y1 = (ydim - 1024) / 2
    y2 = y1 + 1023

    print, 'xdim, ydim: ', xdim, ydim
    print, 'x1, x2, y1, y2: ', x1, x2, y1, y2
    print, 'mag: ', mag
    print, 'oxcen, oycen: ', oxcen, oycen
    print, 'pixrs, opixrs: ', pixrs, opixrs

    IF (xdim GE 1024 AND ydim GE 1024) THEN	$
    BEGIN
        img = img [x1:x2, y1:y2] 			; Crop image: 1024x1024
    END					$
    ELSE					$
    BEGIN
        print, 'invalid image size: ', xdim, ydim
        RETURN
    END

    img   = img > 0
    img   = congrid (img, outdim, outdim)		; Re-size image.

    ; --------------------------------------------------------------------------
    ; Image brightness min/max values.
    ; --------------------------------------------------------------------------

    dispmin = index.dispmin		; display minimum.
    dispmax = index.dispmax		; display maximum.

    ; --------------------------------------------------------------------------
    ; Adjust pixel brightness, using dispmin & dispmax values.
    ; --------------------------------------------------------------------------

    img   = img - 4
    mask_im = where (img LE min (img))		; Clip dim values.
    img [mask_im] = 0  

    imin = min (img, max=imax)				; Ge min/max.

    print, 'imin, imax: ', imin, imax

;    img = bytscl (img, min=imax*0.5, max=imax*0.80)	; Re-scale.

    imin =  700
    imax = 1200
    print, 'imin, imax: ', imin, imax

    img  = bytscl (img, min=imin, max=imax)			; Re-scale.

    ; --------------------------------------------------------------------------
    ; Smooth image.
    ; --------------------------------------------------------------------------

;    img = smooth (img, 3)

    ; --------------------------------------------------------------------------
    ; Set pixels inside occulting disk and outer FOV to zero.
    ; --------------------------------------------------------------------------

    img = disc_mask (img, icolor, outdim, outdim, oxcen, oycen, opixrs)

    ; --------------------------------------------------------------------------
    ; Display image.
    ; --------------------------------------------------------------------------

    tv, img

    ; --------------------------------------------------------------------------
    ; Create outdir, outfn, color_table_code.
    ; --------------------------------------------------------------------------

    outfn = 'mlso_picsf_' + year + month + day + '_' + hours + mins + sec
;    outdir = !swbs.out_dir + 'mlso/' + year + '/' + month + '/' + day + '/'
    outdir = '/archive/swbs/' + 'mlso/' + year + '/' + month + '/' + day + '/'
    data_type = 'I'
    data_name = 'MLSO H-alpha Disc Flat'
    color_table_code = '001' 

    ; --------------------------------------------------------------------------
    ; Solar radius in arcsec.
    ; --------------------------------------------------------------------------

    sol_rad = index.rsun

    ; --------------------------------------------------------------------------
    ; SWB's standard Field Of View (FOV).
    ; --------------------------------------------------------------------------

    swb_fov = sol_rad * outdim / ref_solradius  

    ; --------------------------------------------------------------------------
    ; Calculate the ratio of COR's FOV to the standard SWB's FOV.
    ; --------------------------------------------------------------------------

    pics_fov = index.naxis1 * index.cdelt1 
    fov_ratio = pics_fov / swb_fov

    ; --------------------------------------------------------------------------
    ; Create zip file.
    ; --------------------------------------------------------------------------

    write2zip, outfn, outdir, img, index, data_type, data_name, $
               color_table_code, date, fov_ratio=fov_ratio, /logfile
    ;cd, !swbs.source_dir + 'bin/' + myname
    printf, log, '  processed pics disc image ', list [i]

    ; --------------------------------------------------------------------------

    ENDFOR ;} i
ENDELSE	;} ------- list not empty -------------

; ------------------------------------------------------------------------------
; Say bye-bye.
; ------------------------------------------------------------------------------

spawn, 'date', fulldate   
printf, log, ' <<<< ' + myname + ' finished at ' + fulldate 

; ------------------------------------------------------------------------------
; Close log file, if needed.
; ------------------------------------------------------------------------------

IF (keyword_set (logfile)) THEN free_lun, log   
END
